# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: deployment.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
import bentoml.proto.status_pb2 as status__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='deployment.proto',
  package='bentoml',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x10\x64\x65ployment.proto\x12\x07\x62\x65ntoml\x1a\x19google/protobuf/any.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x0cstatus.proto\"\xc9\t\n\x0e\x44\x65ploymentSpec\x12\x12\n\nbento_name\x18\x01 \x01(\t\x12\x15\n\rbento_version\x18\x02 \x01(\t\x12<\n\x08operator\x18\x03 \x01(\x0e\x32*.bentoml.DeploymentSpec.DeploymentOperator\x12N\n\x16\x63ustom_operator_config\x18\x04 \x01(\x0b\x32,.bentoml.DeploymentSpec.CustomOperatorConfigH\x00\x12T\n\x19sagemaker_operator_config\x18\x05 \x01(\x0b\x32/.bentoml.DeploymentSpec.SageMakerOperatorConfigH\x00\x12U\n\x1a\x61ws_lambda_operator_config\x18\x06 \x01(\x0b\x32/.bentoml.DeploymentSpec.AwsLambdaOperatorConfigH\x00\x12Y\n\x1cgcp_function_operator_config\x18\x07 \x01(\x0b\x32\x31.bentoml.DeploymentSpec.GcpFunctionOperatorConfigH\x00\x12V\n\x1akubernetes_operator_config\x18\x08 \x01(\x0b\x32\x30.bentoml.DeploymentSpec.KubernetesOperatorConfigH\x00\x1a\xb3\x01\n\x14\x43ustomOperatorConfig\x12\x0c\n\x04name\x18\x01 \x01(\t\x12H\n\x06\x63onfig\x18\x02 \x03(\x0b\x32\x38.bentoml.DeploymentSpec.CustomOperatorConfig.ConfigEntry\x1a\x43\n\x0b\x43onfigEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any:\x02\x38\x01\x1aj\n\x17SageMakerOperatorConfig\x12\x0e\n\x06region\x18\x01 \x01(\t\x12\x15\n\rinstance_type\x18\x02 \x01(\t\x12\x16\n\x0einstance_count\x18\x03 \x01(\x05\x12\x10\n\x08\x61pi_name\x18\x04 \x01(\t\x1a;\n\x17\x41wsLambdaOperatorConfig\x12\x0e\n\x06region\x18\x01 \x01(\t\x12\x10\n\x08\x61pi_name\x18\x02 \x01(\t\x1a=\n\x19GcpFunctionOperatorConfig\x12\x0e\n\x06region\x18\x01 \x01(\t\x12\x10\n\x08\x61pi_name\x18\x02 \x01(\t\x1ap\n\x18KubernetesOperatorConfig\x12\x16\n\x0ekube_namespace\x18\x01 \x01(\t\x12\x10\n\x08replicas\x18\x02 \x01(\x05\x12\x14\n\x0cservice_name\x18\x03 \x01(\t\x12\x14\n\x0cservice_type\x18\x04 \x01(\t\"p\n\x12\x44\x65ploymentOperator\x12\t\n\x05UNSET\x10\x00\x12\n\n\x06\x43USTOM\x10\x01\x12\x11\n\rAWS_SAGEMAKER\x10\x02\x12\x0e\n\nAWS_LAMBDA\x10\x03\x12\x10\n\x0cGCP_FUNCTION\x10\x04\x12\x0e\n\nKUBERNETES\x10\x05\x42\x1c\n\x1a\x64\x65ployment_operator_config\"\xa9\x02\n\x0f\x44\x65ploymentState\x12-\n\x05state\x18\x01 \x01(\x0e\x32\x1e.bentoml.DeploymentState.State\x12\x15\n\rerror_message\x18\x02 \x01(\t\x12\x11\n\tinfo_json\x18\x03 \x01(\t\x12-\n\ttimestamp\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\x8d\x01\n\x05State\x12\x0b\n\x07PENDING\x10\x00\x12\x0b\n\x07RUNNING\x10\x01\x12\r\n\tSUCCEEDED\x10\x02\x12\n\n\x06\x46\x41ILED\x10\x03\x12\x0b\n\x07UNKNOWN\x10\x04\x12\r\n\tCOMPLETED\x10\x05\x12\x17\n\x13\x43RASH_LOOP_BACK_OFF\x10\x06\x12\t\n\x05\x45RROR\x10\x07\x12\x0f\n\x0bINACTIVATED\x10\x08\"\xb1\x03\n\nDeployment\x12\x11\n\tnamespace\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12%\n\x04spec\x18\x03 \x01(\x0b\x32\x17.bentoml.DeploymentSpec\x12\'\n\x05state\x18\x04 \x01(\x0b\x32\x18.bentoml.DeploymentState\x12\x39\n\x0b\x61nnotations\x18\x05 \x03(\x0b\x32$.bentoml.Deployment.AnnotationsEntry\x12/\n\x06labels\x18\x06 \x03(\x0b\x32\x1f.bentoml.Deployment.LabelsEntry\x12.\n\ncreated_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x33\n\x0flast_updated_at\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\";\n\x10\x44\x65ploymentStatus\x12\'\n\x05state\x18\x01 \x01(\x0b\x32\x18.bentoml.DeploymentState\"A\n\x16\x41pplyDeploymentRequest\x12\'\n\ndeployment\x18\x01 \x01(\x0b\x32\x13.bentoml.Deployment\"c\n\x17\x41pplyDeploymentResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\x12\'\n\ndeployment\x18\x02 \x01(\x0b\x32\x13.bentoml.Deployment\"[\n\x17\x44\x65leteDeploymentRequest\x12\x17\n\x0f\x64\x65ployment_name\x18\x01 \x01(\t\x12\x11\n\tnamespace\x18\x02 \x01(\t\x12\x14\n\x0c\x66orce_delete\x18\x03 \x01(\x08\";\n\x18\x44\x65leteDeploymentResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\"B\n\x14GetDeploymentRequest\x12\x17\n\x0f\x64\x65ployment_name\x18\x01 \x01(\t\x12\x11\n\tnamespace\x18\x02 \x01(\t\"a\n\x15GetDeploymentResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\x12\'\n\ndeployment\x18\x02 \x01(\x0b\x32\x13.bentoml.Deployment\"G\n\x19\x44\x65scribeDeploymentRequest\x12\x17\n\x0f\x64\x65ployment_name\x18\x01 \x01(\t\x12\x11\n\tnamespace\x18\x02 \x01(\t\"f\n\x1a\x44\x65scribeDeploymentResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\x12\'\n\x05state\x18\x02 \x01(\x0b\x32\x18.bentoml.DeploymentState\"\xc6\x01\n\x16ListDeploymentsRequest\x12\x11\n\tnamespace\x18\x01 \x01(\t\x12\x0e\n\x06offset\x18\x02 \x01(\x05\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\x0e\n\x06\x66ilter\x18\x04 \x01(\t\x12;\n\x06labels\x18\x05 \x03(\x0b\x32+.bentoml.ListDeploymentsRequest.LabelsEntry\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"d\n\x17ListDeploymentsResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\x12(\n\x0b\x64\x65ployments\x18\x02 \x03(\x0b\x32\x13.bentoml.Deploymentb\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,status__pb2.DESCRIPTOR,])



_DEPLOYMENTSPEC_DEPLOYMENTOPERATOR = _descriptor.EnumDescriptor(
  name='DeploymentOperator',
  full_name='bentoml.DeploymentSpec.DeploymentOperator',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNSET', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CUSTOM', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='AWS_SAGEMAKER', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='AWS_LAMBDA', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GCP_FUNCTION', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='KUBERNETES', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1187,
  serialized_end=1299,
)
_sym_db.RegisterEnumDescriptor(_DEPLOYMENTSPEC_DEPLOYMENTOPERATOR)

_DEPLOYMENTSTATE_STATE = _descriptor.EnumDescriptor(
  name='State',
  full_name='bentoml.DeploymentState.State',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PENDING', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RUNNING', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUCCEEDED', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILED', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='COMPLETED', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CRASH_LOOP_BACK_OFF', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=7, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INACTIVATED', index=8, number=8,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1488,
  serialized_end=1629,
)
_sym_db.RegisterEnumDescriptor(_DEPLOYMENTSTATE_STATE)


_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG_CONFIGENTRY = _descriptor.Descriptor(
  name='ConfigEntry',
  full_name='bentoml.DeploymentSpec.CustomOperatorConfig.ConfigEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='bentoml.DeploymentSpec.CustomOperatorConfig.ConfigEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='bentoml.DeploymentSpec.CustomOperatorConfig.ConfigEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=772,
  serialized_end=839,
)

_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG = _descriptor.Descriptor(
  name='CustomOperatorConfig',
  full_name='bentoml.DeploymentSpec.CustomOperatorConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bentoml.DeploymentSpec.CustomOperatorConfig.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config', full_name='bentoml.DeploymentSpec.CustomOperatorConfig.config', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG_CONFIGENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=660,
  serialized_end=839,
)

_DEPLOYMENTSPEC_SAGEMAKEROPERATORCONFIG = _descriptor.Descriptor(
  name='SageMakerOperatorConfig',
  full_name='bentoml.DeploymentSpec.SageMakerOperatorConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='region', full_name='bentoml.DeploymentSpec.SageMakerOperatorConfig.region', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='instance_type', full_name='bentoml.DeploymentSpec.SageMakerOperatorConfig.instance_type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='instance_count', full_name='bentoml.DeploymentSpec.SageMakerOperatorConfig.instance_count', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='api_name', full_name='bentoml.DeploymentSpec.SageMakerOperatorConfig.api_name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=841,
  serialized_end=947,
)

_DEPLOYMENTSPEC_AWSLAMBDAOPERATORCONFIG = _descriptor.Descriptor(
  name='AwsLambdaOperatorConfig',
  full_name='bentoml.DeploymentSpec.AwsLambdaOperatorConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='region', full_name='bentoml.DeploymentSpec.AwsLambdaOperatorConfig.region', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='api_name', full_name='bentoml.DeploymentSpec.AwsLambdaOperatorConfig.api_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=949,
  serialized_end=1008,
)

_DEPLOYMENTSPEC_GCPFUNCTIONOPERATORCONFIG = _descriptor.Descriptor(
  name='GcpFunctionOperatorConfig',
  full_name='bentoml.DeploymentSpec.GcpFunctionOperatorConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='region', full_name='bentoml.DeploymentSpec.GcpFunctionOperatorConfig.region', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='api_name', full_name='bentoml.DeploymentSpec.GcpFunctionOperatorConfig.api_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1010,
  serialized_end=1071,
)

_DEPLOYMENTSPEC_KUBERNETESOPERATORCONFIG = _descriptor.Descriptor(
  name='KubernetesOperatorConfig',
  full_name='bentoml.DeploymentSpec.KubernetesOperatorConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='kube_namespace', full_name='bentoml.DeploymentSpec.KubernetesOperatorConfig.kube_namespace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='replicas', full_name='bentoml.DeploymentSpec.KubernetesOperatorConfig.replicas', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='service_name', full_name='bentoml.DeploymentSpec.KubernetesOperatorConfig.service_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='service_type', full_name='bentoml.DeploymentSpec.KubernetesOperatorConfig.service_type', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1073,
  serialized_end=1185,
)

_DEPLOYMENTSPEC = _descriptor.Descriptor(
  name='DeploymentSpec',
  full_name='bentoml.DeploymentSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bento_name', full_name='bentoml.DeploymentSpec.bento_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bento_version', full_name='bentoml.DeploymentSpec.bento_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='operator', full_name='bentoml.DeploymentSpec.operator', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='custom_operator_config', full_name='bentoml.DeploymentSpec.custom_operator_config', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sagemaker_operator_config', full_name='bentoml.DeploymentSpec.sagemaker_operator_config', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='aws_lambda_operator_config', full_name='bentoml.DeploymentSpec.aws_lambda_operator_config', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gcp_function_operator_config', full_name='bentoml.DeploymentSpec.gcp_function_operator_config', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='kubernetes_operator_config', full_name='bentoml.DeploymentSpec.kubernetes_operator_config', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG, _DEPLOYMENTSPEC_SAGEMAKEROPERATORCONFIG, _DEPLOYMENTSPEC_AWSLAMBDAOPERATORCONFIG, _DEPLOYMENTSPEC_GCPFUNCTIONOPERATORCONFIG, _DEPLOYMENTSPEC_KUBERNETESOPERATORCONFIG, ],
  enum_types=[
    _DEPLOYMENTSPEC_DEPLOYMENTOPERATOR,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='deployment_operator_config', full_name='bentoml.DeploymentSpec.deployment_operator_config',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=104,
  serialized_end=1329,
)


_DEPLOYMENTSTATE = _descriptor.Descriptor(
  name='DeploymentState',
  full_name='bentoml.DeploymentState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='bentoml.DeploymentState.state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error_message', full_name='bentoml.DeploymentState.error_message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info_json', full_name='bentoml.DeploymentState.info_json', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='bentoml.DeploymentState.timestamp', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DEPLOYMENTSTATE_STATE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1332,
  serialized_end=1629,
)


_DEPLOYMENT_ANNOTATIONSENTRY = _descriptor.Descriptor(
  name='AnnotationsEntry',
  full_name='bentoml.Deployment.AnnotationsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='bentoml.Deployment.AnnotationsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='bentoml.Deployment.AnnotationsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1968,
  serialized_end=2018,
)

_DEPLOYMENT_LABELSENTRY = _descriptor.Descriptor(
  name='LabelsEntry',
  full_name='bentoml.Deployment.LabelsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='bentoml.Deployment.LabelsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='bentoml.Deployment.LabelsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2020,
  serialized_end=2065,
)

_DEPLOYMENT = _descriptor.Descriptor(
  name='Deployment',
  full_name='bentoml.Deployment',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespace', full_name='bentoml.Deployment.namespace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='bentoml.Deployment.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='spec', full_name='bentoml.Deployment.spec', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state', full_name='bentoml.Deployment.state', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='annotations', full_name='bentoml.Deployment.annotations', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='labels', full_name='bentoml.Deployment.labels', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='bentoml.Deployment.created_at', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_updated_at', full_name='bentoml.Deployment.last_updated_at', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DEPLOYMENT_ANNOTATIONSENTRY, _DEPLOYMENT_LABELSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1632,
  serialized_end=2065,
)


_DEPLOYMENTSTATUS = _descriptor.Descriptor(
  name='DeploymentStatus',
  full_name='bentoml.DeploymentStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='bentoml.DeploymentStatus.state', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2067,
  serialized_end=2126,
)


_APPLYDEPLOYMENTREQUEST = _descriptor.Descriptor(
  name='ApplyDeploymentRequest',
  full_name='bentoml.ApplyDeploymentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='deployment', full_name='bentoml.ApplyDeploymentRequest.deployment', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2128,
  serialized_end=2193,
)


_APPLYDEPLOYMENTRESPONSE = _descriptor.Descriptor(
  name='ApplyDeploymentResponse',
  full_name='bentoml.ApplyDeploymentResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.ApplyDeploymentResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deployment', full_name='bentoml.ApplyDeploymentResponse.deployment', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2195,
  serialized_end=2294,
)


_DELETEDEPLOYMENTREQUEST = _descriptor.Descriptor(
  name='DeleteDeploymentRequest',
  full_name='bentoml.DeleteDeploymentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='deployment_name', full_name='bentoml.DeleteDeploymentRequest.deployment_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespace', full_name='bentoml.DeleteDeploymentRequest.namespace', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='force_delete', full_name='bentoml.DeleteDeploymentRequest.force_delete', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2296,
  serialized_end=2387,
)


_DELETEDEPLOYMENTRESPONSE = _descriptor.Descriptor(
  name='DeleteDeploymentResponse',
  full_name='bentoml.DeleteDeploymentResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.DeleteDeploymentResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2389,
  serialized_end=2448,
)


_GETDEPLOYMENTREQUEST = _descriptor.Descriptor(
  name='GetDeploymentRequest',
  full_name='bentoml.GetDeploymentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='deployment_name', full_name='bentoml.GetDeploymentRequest.deployment_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespace', full_name='bentoml.GetDeploymentRequest.namespace', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2450,
  serialized_end=2516,
)


_GETDEPLOYMENTRESPONSE = _descriptor.Descriptor(
  name='GetDeploymentResponse',
  full_name='bentoml.GetDeploymentResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.GetDeploymentResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deployment', full_name='bentoml.GetDeploymentResponse.deployment', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2518,
  serialized_end=2615,
)


_DESCRIBEDEPLOYMENTREQUEST = _descriptor.Descriptor(
  name='DescribeDeploymentRequest',
  full_name='bentoml.DescribeDeploymentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='deployment_name', full_name='bentoml.DescribeDeploymentRequest.deployment_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespace', full_name='bentoml.DescribeDeploymentRequest.namespace', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2617,
  serialized_end=2688,
)


_DESCRIBEDEPLOYMENTRESPONSE = _descriptor.Descriptor(
  name='DescribeDeploymentResponse',
  full_name='bentoml.DescribeDeploymentResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.DescribeDeploymentResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state', full_name='bentoml.DescribeDeploymentResponse.state', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2690,
  serialized_end=2792,
)


_LISTDEPLOYMENTSREQUEST_LABELSENTRY = _descriptor.Descriptor(
  name='LabelsEntry',
  full_name='bentoml.ListDeploymentsRequest.LabelsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='bentoml.ListDeploymentsRequest.LabelsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='bentoml.ListDeploymentsRequest.LabelsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2020,
  serialized_end=2065,
)

_LISTDEPLOYMENTSREQUEST = _descriptor.Descriptor(
  name='ListDeploymentsRequest',
  full_name='bentoml.ListDeploymentsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespace', full_name='bentoml.ListDeploymentsRequest.namespace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='offset', full_name='bentoml.ListDeploymentsRequest.offset', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='limit', full_name='bentoml.ListDeploymentsRequest.limit', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='filter', full_name='bentoml.ListDeploymentsRequest.filter', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='labels', full_name='bentoml.ListDeploymentsRequest.labels', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_LISTDEPLOYMENTSREQUEST_LABELSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2795,
  serialized_end=2993,
)


_LISTDEPLOYMENTSRESPONSE = _descriptor.Descriptor(
  name='ListDeploymentsResponse',
  full_name='bentoml.ListDeploymentsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.ListDeploymentsResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deployments', full_name='bentoml.ListDeploymentsResponse.deployments', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2995,
  serialized_end=3095,
)

_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG_CONFIGENTRY.fields_by_name['value'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG_CONFIGENTRY.containing_type = _DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG
_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG.fields_by_name['config'].message_type = _DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG_CONFIGENTRY
_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG.containing_type = _DEPLOYMENTSPEC
_DEPLOYMENTSPEC_SAGEMAKEROPERATORCONFIG.containing_type = _DEPLOYMENTSPEC
_DEPLOYMENTSPEC_AWSLAMBDAOPERATORCONFIG.containing_type = _DEPLOYMENTSPEC
_DEPLOYMENTSPEC_GCPFUNCTIONOPERATORCONFIG.containing_type = _DEPLOYMENTSPEC
_DEPLOYMENTSPEC_KUBERNETESOPERATORCONFIG.containing_type = _DEPLOYMENTSPEC
_DEPLOYMENTSPEC.fields_by_name['operator'].enum_type = _DEPLOYMENTSPEC_DEPLOYMENTOPERATOR
_DEPLOYMENTSPEC.fields_by_name['custom_operator_config'].message_type = _DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG
_DEPLOYMENTSPEC.fields_by_name['sagemaker_operator_config'].message_type = _DEPLOYMENTSPEC_SAGEMAKEROPERATORCONFIG
_DEPLOYMENTSPEC.fields_by_name['aws_lambda_operator_config'].message_type = _DEPLOYMENTSPEC_AWSLAMBDAOPERATORCONFIG
_DEPLOYMENTSPEC.fields_by_name['gcp_function_operator_config'].message_type = _DEPLOYMENTSPEC_GCPFUNCTIONOPERATORCONFIG
_DEPLOYMENTSPEC.fields_by_name['kubernetes_operator_config'].message_type = _DEPLOYMENTSPEC_KUBERNETESOPERATORCONFIG
_DEPLOYMENTSPEC_DEPLOYMENTOPERATOR.containing_type = _DEPLOYMENTSPEC
_DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config'].fields.append(
  _DEPLOYMENTSPEC.fields_by_name['custom_operator_config'])
_DEPLOYMENTSPEC.fields_by_name['custom_operator_config'].containing_oneof = _DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config']
_DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config'].fields.append(
  _DEPLOYMENTSPEC.fields_by_name['sagemaker_operator_config'])
_DEPLOYMENTSPEC.fields_by_name['sagemaker_operator_config'].containing_oneof = _DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config']
_DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config'].fields.append(
  _DEPLOYMENTSPEC.fields_by_name['aws_lambda_operator_config'])
_DEPLOYMENTSPEC.fields_by_name['aws_lambda_operator_config'].containing_oneof = _DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config']
_DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config'].fields.append(
  _DEPLOYMENTSPEC.fields_by_name['gcp_function_operator_config'])
_DEPLOYMENTSPEC.fields_by_name['gcp_function_operator_config'].containing_oneof = _DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config']
_DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config'].fields.append(
  _DEPLOYMENTSPEC.fields_by_name['kubernetes_operator_config'])
_DEPLOYMENTSPEC.fields_by_name['kubernetes_operator_config'].containing_oneof = _DEPLOYMENTSPEC.oneofs_by_name['deployment_operator_config']
_DEPLOYMENTSTATE.fields_by_name['state'].enum_type = _DEPLOYMENTSTATE_STATE
_DEPLOYMENTSTATE.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_DEPLOYMENTSTATE_STATE.containing_type = _DEPLOYMENTSTATE
_DEPLOYMENT_ANNOTATIONSENTRY.containing_type = _DEPLOYMENT
_DEPLOYMENT_LABELSENTRY.containing_type = _DEPLOYMENT
_DEPLOYMENT.fields_by_name['spec'].message_type = _DEPLOYMENTSPEC
_DEPLOYMENT.fields_by_name['state'].message_type = _DEPLOYMENTSTATE
_DEPLOYMENT.fields_by_name['annotations'].message_type = _DEPLOYMENT_ANNOTATIONSENTRY
_DEPLOYMENT.fields_by_name['labels'].message_type = _DEPLOYMENT_LABELSENTRY
_DEPLOYMENT.fields_by_name['created_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_DEPLOYMENT.fields_by_name['last_updated_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_DEPLOYMENTSTATUS.fields_by_name['state'].message_type = _DEPLOYMENTSTATE
_APPLYDEPLOYMENTREQUEST.fields_by_name['deployment'].message_type = _DEPLOYMENT
_APPLYDEPLOYMENTRESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_APPLYDEPLOYMENTRESPONSE.fields_by_name['deployment'].message_type = _DEPLOYMENT
_DELETEDEPLOYMENTRESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_GETDEPLOYMENTRESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_GETDEPLOYMENTRESPONSE.fields_by_name['deployment'].message_type = _DEPLOYMENT
_DESCRIBEDEPLOYMENTRESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_DESCRIBEDEPLOYMENTRESPONSE.fields_by_name['state'].message_type = _DEPLOYMENTSTATE
_LISTDEPLOYMENTSREQUEST_LABELSENTRY.containing_type = _LISTDEPLOYMENTSREQUEST
_LISTDEPLOYMENTSREQUEST.fields_by_name['labels'].message_type = _LISTDEPLOYMENTSREQUEST_LABELSENTRY
_LISTDEPLOYMENTSRESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_LISTDEPLOYMENTSRESPONSE.fields_by_name['deployments'].message_type = _DEPLOYMENT
DESCRIPTOR.message_types_by_name['DeploymentSpec'] = _DEPLOYMENTSPEC
DESCRIPTOR.message_types_by_name['DeploymentState'] = _DEPLOYMENTSTATE
DESCRIPTOR.message_types_by_name['Deployment'] = _DEPLOYMENT
DESCRIPTOR.message_types_by_name['DeploymentStatus'] = _DEPLOYMENTSTATUS
DESCRIPTOR.message_types_by_name['ApplyDeploymentRequest'] = _APPLYDEPLOYMENTREQUEST
DESCRIPTOR.message_types_by_name['ApplyDeploymentResponse'] = _APPLYDEPLOYMENTRESPONSE
DESCRIPTOR.message_types_by_name['DeleteDeploymentRequest'] = _DELETEDEPLOYMENTREQUEST
DESCRIPTOR.message_types_by_name['DeleteDeploymentResponse'] = _DELETEDEPLOYMENTRESPONSE
DESCRIPTOR.message_types_by_name['GetDeploymentRequest'] = _GETDEPLOYMENTREQUEST
DESCRIPTOR.message_types_by_name['GetDeploymentResponse'] = _GETDEPLOYMENTRESPONSE
DESCRIPTOR.message_types_by_name['DescribeDeploymentRequest'] = _DESCRIBEDEPLOYMENTREQUEST
DESCRIPTOR.message_types_by_name['DescribeDeploymentResponse'] = _DESCRIBEDEPLOYMENTRESPONSE
DESCRIPTOR.message_types_by_name['ListDeploymentsRequest'] = _LISTDEPLOYMENTSREQUEST
DESCRIPTOR.message_types_by_name['ListDeploymentsResponse'] = _LISTDEPLOYMENTSRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DeploymentSpec = _reflection.GeneratedProtocolMessageType('DeploymentSpec', (_message.Message,), {

  'CustomOperatorConfig' : _reflection.GeneratedProtocolMessageType('CustomOperatorConfig', (_message.Message,), {

    'ConfigEntry' : _reflection.GeneratedProtocolMessageType('ConfigEntry', (_message.Message,), {
      'DESCRIPTOR' : _DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG_CONFIGENTRY,
      '__module__' : 'deployment_pb2'
      # @@protoc_insertion_point(class_scope:bentoml.DeploymentSpec.CustomOperatorConfig.ConfigEntry)
      })
    ,
    'DESCRIPTOR' : _DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.DeploymentSpec.CustomOperatorConfig)
    })
  ,

  'SageMakerOperatorConfig' : _reflection.GeneratedProtocolMessageType('SageMakerOperatorConfig', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYMENTSPEC_SAGEMAKEROPERATORCONFIG,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.DeploymentSpec.SageMakerOperatorConfig)
    })
  ,

  'AwsLambdaOperatorConfig' : _reflection.GeneratedProtocolMessageType('AwsLambdaOperatorConfig', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYMENTSPEC_AWSLAMBDAOPERATORCONFIG,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.DeploymentSpec.AwsLambdaOperatorConfig)
    })
  ,

  'GcpFunctionOperatorConfig' : _reflection.GeneratedProtocolMessageType('GcpFunctionOperatorConfig', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYMENTSPEC_GCPFUNCTIONOPERATORCONFIG,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.DeploymentSpec.GcpFunctionOperatorConfig)
    })
  ,

  'KubernetesOperatorConfig' : _reflection.GeneratedProtocolMessageType('KubernetesOperatorConfig', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYMENTSPEC_KUBERNETESOPERATORCONFIG,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.DeploymentSpec.KubernetesOperatorConfig)
    })
  ,
  'DESCRIPTOR' : _DEPLOYMENTSPEC,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DeploymentSpec)
  })
_sym_db.RegisterMessage(DeploymentSpec)
_sym_db.RegisterMessage(DeploymentSpec.CustomOperatorConfig)
_sym_db.RegisterMessage(DeploymentSpec.CustomOperatorConfig.ConfigEntry)
_sym_db.RegisterMessage(DeploymentSpec.SageMakerOperatorConfig)
_sym_db.RegisterMessage(DeploymentSpec.AwsLambdaOperatorConfig)
_sym_db.RegisterMessage(DeploymentSpec.GcpFunctionOperatorConfig)
_sym_db.RegisterMessage(DeploymentSpec.KubernetesOperatorConfig)

DeploymentState = _reflection.GeneratedProtocolMessageType('DeploymentState', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYMENTSTATE,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DeploymentState)
  })
_sym_db.RegisterMessage(DeploymentState)

Deployment = _reflection.GeneratedProtocolMessageType('Deployment', (_message.Message,), {

  'AnnotationsEntry' : _reflection.GeneratedProtocolMessageType('AnnotationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYMENT_ANNOTATIONSENTRY,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.Deployment.AnnotationsEntry)
    })
  ,

  'LabelsEntry' : _reflection.GeneratedProtocolMessageType('LabelsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYMENT_LABELSENTRY,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.Deployment.LabelsEntry)
    })
  ,
  'DESCRIPTOR' : _DEPLOYMENT,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.Deployment)
  })
_sym_db.RegisterMessage(Deployment)
_sym_db.RegisterMessage(Deployment.AnnotationsEntry)
_sym_db.RegisterMessage(Deployment.LabelsEntry)

DeploymentStatus = _reflection.GeneratedProtocolMessageType('DeploymentStatus', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYMENTSTATUS,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DeploymentStatus)
  })
_sym_db.RegisterMessage(DeploymentStatus)

ApplyDeploymentRequest = _reflection.GeneratedProtocolMessageType('ApplyDeploymentRequest', (_message.Message,), {
  'DESCRIPTOR' : _APPLYDEPLOYMENTREQUEST,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.ApplyDeploymentRequest)
  })
_sym_db.RegisterMessage(ApplyDeploymentRequest)

ApplyDeploymentResponse = _reflection.GeneratedProtocolMessageType('ApplyDeploymentResponse', (_message.Message,), {
  'DESCRIPTOR' : _APPLYDEPLOYMENTRESPONSE,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.ApplyDeploymentResponse)
  })
_sym_db.RegisterMessage(ApplyDeploymentResponse)

DeleteDeploymentRequest = _reflection.GeneratedProtocolMessageType('DeleteDeploymentRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEDEPLOYMENTREQUEST,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DeleteDeploymentRequest)
  })
_sym_db.RegisterMessage(DeleteDeploymentRequest)

DeleteDeploymentResponse = _reflection.GeneratedProtocolMessageType('DeleteDeploymentResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEDEPLOYMENTRESPONSE,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DeleteDeploymentResponse)
  })
_sym_db.RegisterMessage(DeleteDeploymentResponse)

GetDeploymentRequest = _reflection.GeneratedProtocolMessageType('GetDeploymentRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETDEPLOYMENTREQUEST,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.GetDeploymentRequest)
  })
_sym_db.RegisterMessage(GetDeploymentRequest)

GetDeploymentResponse = _reflection.GeneratedProtocolMessageType('GetDeploymentResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETDEPLOYMENTRESPONSE,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.GetDeploymentResponse)
  })
_sym_db.RegisterMessage(GetDeploymentResponse)

DescribeDeploymentRequest = _reflection.GeneratedProtocolMessageType('DescribeDeploymentRequest', (_message.Message,), {
  'DESCRIPTOR' : _DESCRIBEDEPLOYMENTREQUEST,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DescribeDeploymentRequest)
  })
_sym_db.RegisterMessage(DescribeDeploymentRequest)

DescribeDeploymentResponse = _reflection.GeneratedProtocolMessageType('DescribeDeploymentResponse', (_message.Message,), {
  'DESCRIPTOR' : _DESCRIBEDEPLOYMENTRESPONSE,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DescribeDeploymentResponse)
  })
_sym_db.RegisterMessage(DescribeDeploymentResponse)

ListDeploymentsRequest = _reflection.GeneratedProtocolMessageType('ListDeploymentsRequest', (_message.Message,), {

  'LabelsEntry' : _reflection.GeneratedProtocolMessageType('LabelsEntry', (_message.Message,), {
    'DESCRIPTOR' : _LISTDEPLOYMENTSREQUEST_LABELSENTRY,
    '__module__' : 'deployment_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.ListDeploymentsRequest.LabelsEntry)
    })
  ,
  'DESCRIPTOR' : _LISTDEPLOYMENTSREQUEST,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.ListDeploymentsRequest)
  })
_sym_db.RegisterMessage(ListDeploymentsRequest)
_sym_db.RegisterMessage(ListDeploymentsRequest.LabelsEntry)

ListDeploymentsResponse = _reflection.GeneratedProtocolMessageType('ListDeploymentsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTDEPLOYMENTSRESPONSE,
  '__module__' : 'deployment_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.ListDeploymentsResponse)
  })
_sym_db.RegisterMessage(ListDeploymentsResponse)


_DEPLOYMENTSPEC_CUSTOMOPERATORCONFIG_CONFIGENTRY._options = None
_DEPLOYMENT_ANNOTATIONSENTRY._options = None
_DEPLOYMENT_LABELSENTRY._options = None
_LISTDEPLOYMENTSREQUEST_LABELSENTRY._options = None
# @@protoc_insertion_point(module_scope)
